@@ -5,6 +5,7 @@ import logging
 import sys
 import time
 from code import InteractiveConsole
+from collections.abc import Awaitable
 from contextlib import contextmanager
 from types import ModuleType
 
@@ -67,7 +68,10 @@ class Runner:
         code_obj = self.pre_run(source_code, mode, top_level_await=top_level_await)
         with self._execute_context(source_code):
             if code_obj:
-                return await self.execute(code_obj, source_code, mode)
+                result = self.execute(code_obj, source_code, mode)
+                while isinstance(result, Awaitable):
+                    result = await result
+                return result
 
     def serialize_traceback(self, exc, source_code):  # noqa
         raise NotImplementedError  # pragma: no cover
@@ -130,17 +134,18 @@ class PatchedStdinRunner(Runner):  # noqa
         super().reset()
         self.line = ""
 
-    def non_str_input(self):
-        raise TypeError(f"Callback for input should return str, not {type(self.line).__name__}")
+    def non_str_input(self, value):
+        raise TypeError(f"Callback for input should return str, not {type(value).__name__}")
 
     def readline(self, n=-1, prompt=""):
         if not self.line and n:
-            self.line = self.callback("input", prompt=prompt)
-            if not isinstance(self.line, str):
-                self.non_str_input()
-            if not self.line.endswith("\n"):
-                self.line += "\n"
-            self.output("input", self.line)
+            value = self.callback("input", prompt=prompt)
+            if not isinstance(value, str):
+                value = self.non_str_input(value) or ""
+            if not value.endswith("\n"):
+                value += "\n"
+            self.output("input", value)
+            self.line = value
 
         if n < 0 or n > len(self.line):
             n = len(self.line)
@@ -159,4 +164,8 @@ class PatchedSleepRunner(Runner):  # noqa
         return super().execute(*args, **kwargs)
 
     def sleep(self, seconds):
+        if not isinstance(seconds, (int, float)):
+            raise TypeError(f"an integer is required (got type {type(seconds).__name__})")
+        if not seconds >= 0:
+            raise ValueError("sleep length must be non-negative")
         return self.callback("sleep", seconds=seconds)
